@@ -5,14 +5,14 @@
  *
  *      Description:  Benchmarking framework for likwid-bench
  *
- *      Version:   4.0
- *      Released:  16.6.2015
+ *      Version:   4.1
+ *      Released:  13.6.2016
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *               Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -36,7 +36,6 @@
 #include <sys/syscall.h>
 #include <string.h>
 #include <sched.h>
-//#include <types.h>
 #include <unistd.h>
 
 #include <allocator.h>
@@ -49,30 +48,20 @@
 
 /* #####   MACROS  -  LOCAL TO THIS SOURCE FILE   ######################### */
 
-//#define BARRIER pthread_barrier_wait(&threads_barrier)
 #define BARRIER   barrier_synchronize(&barr)
 
-#ifdef PERFMON
-#define START_PERFMON likwid_markerStartRegion("bench");
-#define STOP_PERFMON  likwid_markerStopRegion("bench");
-#define LIKWID_THREAD_INIT  likwid_markerThreadInit();
-#else
-#define START_PERFMON
-#define STOP_PERFMON
-#define LIKWID_THREAD_INIT
-#endif
 
 #define EXECUTE(func)   \
     BARRIER; \
+    LIKWID_MARKER_START("bench");  \
     timer_start(&time); \
-    START_PERFMON  \
     for (i=0; i<myData->iter; i++) \
     {   \
         func; \
     } \
     BARRIER; \
-    STOP_PERFMON  \
     timer_stop(&time); \
+    LIKWID_MARKER_STOP("bench");  \
     data->cycles = timer_printCycles(&time); \
     BARRIER
 
@@ -85,7 +74,8 @@ void* runTest(void* arg)
 {
     int threadId;
     int offset;
-    size_t size, allSize;
+    size_t size;
+    size_t vecsize;
     size_t i;
     BarrierData barr;
     ThreadData* data;
@@ -100,22 +90,59 @@ void* runTest(void* arg)
     barrier_registerThread(&barr, 0, data->globalThreadId);
 
     /* Prepare ptrs for thread */
-    allSize = myData->size;
+    vecsize = myData->size;
     size = myData->size / data->numberOfThreads;
-    size -= (size%myData->test->stride);
-    offset = data->threadId * size;
     myData->size = size;
+    size -= (size % myData->test->stride);
+    offset = data->threadId * size;
+    
+
+    switch ( myData->test->type )
+    {
+        case SINGLE:
+            {
+                float* sptr;
+                for (i=0; i <  myData->test->streams; i++)
+                {
+                    sptr = (float*) myData->streams[i];
+                    sptr +=  offset;
+                    myData->streams[i] = (float*) sptr;
+                }
+            }
+            break;
+        case INT:
+            {
+                int* sptr;
+                for (i=0; i <  myData->test->streams; i++)
+                {
+                    sptr = (int*) myData->streams[i];
+                    sptr +=  offset;
+                    myData->streams[i] = (int*) sptr;
+                }
+            }
+            break;
+        case DOUBLE:
+            {
+                double* dptr;
+                for (i=0; i <  myData->test->streams; i++)
+                {
+                    dptr = (double*) myData->streams[i];
+                    dptr +=  offset;
+                    myData->streams[i] = (double*) dptr;
+                }
+            }
+            break;
+    }
 
 
     /* pin the thread */
     likwid_pinThread(myData->processors[threadId]);
-
     printf("Group: %d Thread %d Global Thread %d running on core %d - Vector length %llu Offset %d\n",
             data->groupId,
             threadId,
             data->globalThreadId,
             affinity_threadGetProcessorId(),
-            LLU_CAST allSize,
+            LLU_CAST vecsize,
             offset);
     BARRIER;
 
@@ -407,34 +434,34 @@ void* runTest(void* arg)
         default:
             break;
     }
+    free(barr.index);
     pthread_exit(NULL);
 }
 
 
 #define MEASURE(func) \
-    if (data->globalThreadId == 0) \
+    iterations = 8; \
+    while (1) \
     { \
         timer_start(&time); \
-        i = 0; \
-        for (i=0; i < SIZE_MAX; i++) \
+        for (i=0;i<iterations;i++) \
         { \
             func; \
-            timer_stop(&time); \
-            if (timer_print(&time) >= (double)data->data.min_runtime) \
-                break; \
         } \
-        iterations = i;  \
+        timer_stop(&time); \
+        if (timer_print(&time) < (double)data->data.min_runtime) \
+            iterations = iterations << 1; \
+        else \
+            break; \
     } \
-    BARRIER;
 
 
-void* getIter(void* arg)
+void* getIterSingle(void* arg)
 {
-    int threadId;
-    int offset;
-    size_t size;
+    int threadId = 0;
+    int offset = 0;
+    size_t size = 0;
     size_t i;
-    BarrierData barr;
     ThreadData* data;
     ThreadUserData* myData;
     TimerData time;
@@ -445,45 +472,12 @@ void* getIter(void* arg)
     myData = &(data->data);
     func = myData->test->kernel;
     threadId = data->threadId;
-    barrier_registerThread(&barr, 0, data->globalThreadId);
 
-
-    /* Prepare ptrs for thread */
-    size = myData->size / data->numberOfThreads;
-    size -= (size%myData->test->stride);
-    offset = data->threadId * size;
-
-    switch ( myData->test->type )
-    {
-        case SINGLE:
-            {
-                float* sptr;
-                for (i=0; i <  myData->test->streams; i++)
-                {
-                    sptr = (float*) myData->streams[i];
-                    sptr +=  offset;
-                    myData->streams[i] = (float*) sptr;
-                }
-            }
-            break;
-        case DOUBLE:
-            {
-                double* dptr;
-                for (i=0; i <  myData->test->streams; i++)
-                {
-                    dptr = (double*) myData->streams[i];
-                    dptr +=  offset;
-                    myData->streams[i] = (double*) dptr;
-                }
-            }
-            break;
-    }
-
-    /* pin the thread */
+    size = myData->size - (myData->size % myData->test->stride);
     likwid_pinThread(myData->processors[threadId]);
-#ifdef PERFMON
-    LIKWID_THREAD_INIT;
-    BARRIER;
+
+#ifdef DEBUG_LIKWID
+    printf("Automatic iteration count detection:");
 #endif
 
     switch ( myData->test->streams ) {
@@ -766,7 +760,11 @@ void* getIter(void* arg)
         default:
             break;
     }
-
     data->data.iter = iterations;
-    pthread_exit(NULL);
+#ifdef DEBUG_LIKWID
+    printf(" %d iterations per thread\n", iterations);
+    if (iterations < MIN_ITERATIONS)
+        printf("Sanitizing iterations count per thread to %d\n",MIN_ITERATIONS);
+#endif
+    return NULL;
 }
