@@ -1,4 +1,4 @@
-#!<PREFIX>/bin/likwid-lua
+#!<INSTALLED_BINPREFIX>/likwid-lua
 --[[
  * =======================================================================================
  *
@@ -6,13 +6,13 @@
  *
  *      Description:  A monitoring daemon for hardware performance counters.
  *
- *      Version:   4.0
- *      Released:  16.6.2015
+ *      Version:   4.1
+ *      Released:  13.6.2016
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -29,21 +29,23 @@
  * =======================================================================================
 ]]
 
-package.path = '<PREFIX>/share/lua/?.lua;' .. package.path
+package.path = '<INSTALLED_PREFIX>/share/lua/?.lua;' .. package.path
 local likwid = require("likwid")
 
+local base_groupPath = "<INSTALLED_PREFIX>/share/likwid/mongroups"
 dconfig = {}
 dconfig["groupStrings"] ={}
 dconfig["groupData"] ={}
 dconfig["accessmode"] = 1
 dconfig["duration"] = 1
-dconfig["groupPath"] = "<PREFIX>/share/likwid/mongroups"
+dconfig["groupPath"] = ""
 dconfig["logPath"] = nil
 dconfig["logStyle"] = "log"
 dconfig["gmetric"] = false
 dconfig["gmetricPath"] = "gmetric"
 dconfig["gmetricConfig"] = nil
 dconfig["gmetricHasUnit"] = false
+dconfig["gmetricHasGroup"] = false
 dconfig["rrd"] = false
 dconfig["rrdPath"] = "."
 dconfig["syslog"] = false
@@ -207,12 +209,11 @@ local function calc_max(key, results)
 end
 
 local function check_logfile()
-    local g = io.popen("ls "..dconfig["logPath"], "r")
-    if g == nil then
+    local g = os.execute("cd "..dconfig["logPath"], "r")
+    if g == false then
         print("Logfile path".. dconfig["logPath"].. " does not exist.")
         return false
     end
-    g:close()
     return true
 end
 
@@ -282,6 +283,9 @@ local function check_gmetric()
     if msg:match("units=") then
         dconfig["gmetricHasUnit"] = true
     end
+    if msg:match("group=") then
+        dconfig["gmetricHasGroup"] = true
+    end
     f:close()
     return true
 end
@@ -296,7 +300,7 @@ local function gmetric(gdata, results)
         table.insert(execList, "-c")
         table.insert(execList, dconfig["gmetricConfig"])
     end
-    if gdata["GroupString"] ~= gdata["EventString"] then
+    if dconfig["gmetricHasGroup"] and gdata["GroupString"] ~= gdata["EventString"] then
         table.insert(execList, "-g")
         table.insert(execList, gdata["GroupString"])
     end
@@ -304,29 +308,28 @@ local function gmetric(gdata, results)
         local execStr = table.concat(execList, " ")
         if k ~= "Timestamp" then
             execStr = execStr .. " -t double "
-        else
-            execStr = execStr .. " -t string "
+
+            local name = k
+            local unit = nil
+            local s,e = k:find("%[")
+            if s ~= nil then
+                name = k:sub(0,s-2):gsub("^%s*(.-)%s*$", "%1")
+                unit = k:sub(s+1,k:len()-1):gsub("^%s*(.-)%s*$", "%1")
+            end
+            execStr = execStr .. " --name=\"" .. name .."\""
+            if dconfig["gmetricHasUnit"] and unit ~= nil then
+                execStr = execStr .. " --units=\"" .. unit .."\""
+            end
+            local value = tonumber(v)
+            if v ~= nil and value ~= nil then
+                execStr = execStr .. " --value=\"" .. string.format("%f", value) .."\""
+            elseif v ~= nil then
+                execStr = execStr .. " --value=\"" .. tostring(v) .."\""
+            else
+                execStr = execStr .. " --value=\"0\""
+            end
+            os.execute(execStr)
         end
-        local name = k
-        local unit = nil
-        local s,e = k:find("%[")
-        if s ~= nil then
-            name = k:sub(0,s-1):gsub("^%s*(.-)%s*$", "%1")
-            unit = k:sub(s+1,k:len()-1):gsub("^%s*(.-)%s*$", "%1")
-        end
-        execStr = execStr .. " --name=\"" .. name .."\""
-        if dconfig["gmetricHasUnit"] and unit ~= nil then
-            execStr = execStr .. " --units=\"" .. unit .."\""
-        end
-        local value = tonumber(v)
-        if v ~= nil and value ~= nil then
-            execStr = execStr .. " --value=\"" .. string.format("%f", value) .."\""
-        elseif value ~= nil then
-            execStr = execStr .. " --value=\"" .. tostring(v) .."\""
-        else
-            execStr = execStr .. " --value=\"0\""
-        end
-        os.execute(execStr)
     end
 end
 
@@ -394,11 +397,21 @@ end
 -- Get architectural information for the current system
 local cpuinfo = likwid.getCpuInfo()
 local cputopo = likwid.getCpuTopology()
+local affinity = likwid.getAffinityInfo()
 -- Read LIKWID configuration file, mainly to avoid topology lookup
 local config = likwid.getConfiguration()
 -- Read LIKWID daemon configuration file
 read_daemon_config(arg[1])
 
+-- Set force mode, we are monitoring exclusively
+likwid.setenv("LIKWID_FORCE","1")
+
+if dconfig["groupPath"] ~= "" then
+    likwid.setGroupPath(dconfig["groupPath"])
+else
+    dconfig["groupPath"] = base_groupPath
+end
+
 if #dconfig["groupStrings"] == 0 then
     print("No monitoring groups defined, exiting...")
     os.exit(1)
@@ -440,8 +453,8 @@ end
 
 -- Add all cpus to the cpulist
 local cpulist = {}
-for i, thread in pairs(cputopo["threadPool"]) do
-    table.insert(cpulist, thread["apicId"])
+for i=0, cputopo["numHWThreads"]-1 do
+    table.insert(cpulist, cputopo["threadPool"][i]["apicId"])
 end
 
 -- Select access mode to msr devices, try configuration file first
@@ -456,91 +469,64 @@ end
 -- Select group directory for monitoring
 likwid.groupfolder = dconfig["groupPath"]
 
--- Evaluate eventSet given on commandline. If it's a group, resolve to events
-for k,v in pairs(dconfig["groupStrings"]) do
-    local gdata = nil
-    gdata = likwid.get_groupdata(v)
-    if gdata ~= nil then
-        table.insert(dconfig["groupData"], gdata)
-    end
-end
-if #dconfig["groupData"] == 0 then
-    print("None of the event strings can be added for current architecture.")
-    os.exit(1)
-end
-
+power = likwid.getPowerInfo()
 -- Initialize likwid perfctr
 likwid.init(cputopo["numHWThreads"], cpulist)
-for k,v in pairs(dconfig["groupData"]) do
-    local groupID = likwid.addEventSet(v["EventString"])
+for k,v in pairs(dconfig["groupStrings"]) do
+    local groupID = likwid.addEventSet(v)
+    table.insert(dconfig["groupData"], groupID, v)
     if dconfig["rrd"] then
-        create_rrd(#dconfig["groupData"], dconfig["duration"], v)
+        create_rrd(#dconfig["groupStrings"], dconfig["duration"], v)
     end
 end
+
 likwid.catchSignal()
 while likwid.getSignalState() == 0 do
 
-    for groupID,gdata in pairs(dconfig["groupData"]) do
+    for groupID,gname in pairs(dconfig["groupData"]) do
         local old_mtime = likwid_getRuntimeOfGroup(groupID)
         local cur_time = os.time()
         likwid.setupCounters(groupID)
 
         -- Perform the measurement
         likwid.startCounters()
-        sleep(dconfig["duration"])
+        likwid.sleep(dconfig["duration"] * 1E6)
         likwid.stopCounters()
 
-        -- temporal array collecting counter to values for each thread for metric calculation
-        threadResults = {}
-        local mtime = likwid_getRuntimeOfGroup(groupID)
-        local clock = likwid_getCpuClock();
 
-        for event=1, likwid.getNumberOfEvents(groupID) do
-            for thread=1, likwid.getNumberOfThreads() do
-                if threadResults[thread] == nil then
-                    threadResults[thread] = {}
-                end
-                threadResults[thread]["time"] = mtime - old_mtime
-                threadResults[thread]["inverseClock"] = 1.0/clock;
-                local result = likwid.getResult(groupID, event, thread)
-                if threadResults[thread][gdata["Events"][event]["Counter"]] == nil then
-                    threadResults[thread][gdata["Events"][event]["Counter"]] = result
-                end
-            end
-        end
-
-
-        if gdata["Metrics"] then
+        if likwid.getNumberOfMetrics(groupID) > 0 then
             local threadOutput = {}
-            for i, metric in pairs(gdata["Metrics"]) do
+            for i=1, likwid.getNumberOfMetrics(groupID) do
+                local metricdesc = likwid.getNameOfMetric(groupID, i)
                 for thread=1, likwid.getNumberOfThreads() do
                     if threadOutput[thread] == nil then
                         threadOutput[thread] = {}
                     end
-                    local result = likwid.calculate_metric(metric["formula"], threadResults[thread])
-                    threadOutput[thread][metric["description"]] = result
+                    --local result = likwid.calculate_metric(metric["formula"], threadResults[thread])
+                    threadOutput[thread][metricdesc] = likwid.getLastMetric(groupID, i, thread)
                 end
             end
             output = {}
             output["Timestamp"] = os.date("%m/%d/%Y_%X",cur_time)
-            for i, metric in pairs(gdata["Metrics"]) do
-                itemlist = likwid.stringsplit(metric["description"], "%s+", nil, "%s+")
+            for i=1, likwid.getNumberOfMetrics(groupID) do
+                local metricdesc = likwid.getNameOfMetric(groupID, i)
+                itemlist = likwid.stringsplit(metricdesc, "%s+", nil, "%s+")
                 func = itemlist[1]
                 table.remove(itemlist, 1)
                 desc = table.concat(itemlist," ")
                 if func == "AVG" then
-                    output[metric["description"]] = calc_avg(metric["description"], threadOutput)
+                    output[metricdesc:gsub(" ","_")] = calc_avg(metricdesc, threadOutput)
                 elseif func == "SUM" then
-                    output[metric["description"]] = calc_sum(metric["description"], threadOutput)
+                    output[metricdesc:gsub(" ","_")] = calc_sum(metricdesc, threadOutput)
                 elseif func == "MIN" then
-                    output[metric["description"]] = calc_min(metric["description"], threadOutput)
+                    output[metricdesc:gsub(" ","_")] = calc_min(metricdesc, threadOutput)
                 elseif func == "MAX" then
-                    output[metric["description"]] = calc_max(metric["description"], threadOutput)
+                    output[metricdesc:gsub(" ","_")] = calc_max(metricdesc, threadOutput)
                 elseif func == "ONCE" then
-                    output[metric["description"]] = threadOutput[1][metric["description"]]
+                    output[metricdesc:gsub(" ","_")] = threadOutput[1][metricdesc]
                 else
                     for thread=1, likwid.getNumberOfThreads() do
-                        output["T"..cpulist[thread] .. " " .. metric["description"]] = threadOutput[thread][metric["description"]]
+                        output["T"..cpulist[thread] .. "_" .. metricdesc] = threadOutput[thread][metricdesc]
                     end
                 end
             end
