@@ -1,4 +1,4 @@
-#!<PREFIX>/bin/likwid-lua
+#!<INSTALLED_BINPREFIX>/likwid-lua
 --[[
  * =======================================================================================
  *
@@ -6,13 +6,13 @@
  *
  *      Description:  A application to set the CPU frequency of CPU cores and domains.
  *
- *      Version:   4.0
- *      Released:  16.6.2015
+ *      Version:   4.1
+ *      Released:  13.6.2016
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -29,56 +29,123 @@
  * =======================================================================================
 ]]
 
-package.path = '<PREFIX>/share/lua/?.lua;' .. package.path
+package.path = '<INSTALLED_PREFIX>/share/lua/?.lua;' .. package.path
 
 local likwid = require("likwid")
 
+print_stdout = print
+print_stderr = function(...) for k,v in pairs({...}) do io.stderr:write(v .. "\n") end end
+
 sys_base_path = "/sys/devices/system/cpu"
-set_command = "<PREFIX>/sbin/likwid-setFreq"
+set_command = "<INSTALLED_PREFIX>/sbin/likwid-setFreq"
 
 
 function version()
-    print(string.format("likwid-setFrequencies --  Version %d.%d",likwid.version,likwid.release))
+    print_stdout(string.format("likwid-setFrequencies --  Version %d.%d",likwid.version,likwid.release))
 end
 
 function usage()
     version()
-    print("A tool to adjust frequencies and governors on x86 CPUs.\n")
-    print("Options:")
-    print("-h\t Help message")
-    print("-v\t Version information")
-    print("-c dom\t Likwid thread domain which to apply settings (default are all CPUs)")
-    print("\t See likwid-pin -h for details")
-    print("-g gov\t Set governor (" .. table.concat(getAvailGovs(nil), ", ") .. ") (set to ondemand if omitted)")
-    print("-f freq\t Set fixed frequency, implicitly sets userspace governor")
-    print("-p\t Print current frequencies")
-    print("-l\t List available frequencies")
-    print("-m\t List available governors")
+    print_stdout("A tool to adjust frequencies and governors on x86 CPUs.\n")
+    print_stdout("Options:")
+    print_stdout("-h\t Help message")
+    print_stdout("-v\t Version information")
+    print_stdout("-c dom\t Likwid thread domain which to apply settings (default are all CPUs)")
+    print_stdout("\t See likwid-pin -h for details")
+    print_stdout("-g gov\t Set governor (" .. table.concat(getAvailGovs(nil), ", ") .. ") (set to ondemand if omitted)")
+    print_stdout("-f freq\t Set fixed frequency, implicitly sets userspace governor")
+    print_stdout("-p\t Print current frequencies")
+    print_stdout("-l\t List available frequencies")
+    print_stdout("-m\t List available governors")
 end
 
+function getCurrentMinFreq(cpuid)
+    local min = 10000000
+    if cpuid == nil or cpuid < 0 then
+        for cpuid=0,topo["numHWThreads"]-1 do
+            fp = io.open(sys_base_path .. "/" .. string.format("cpu%d",cpuid) .. "/cpufreq/scaling_min_freq")
+            if verbosity == 3 then
+                print_stdout("Reading "..sys_base_path .. "/" .. string.format("cpu%d",cpuid) .. "/cpufreq/scaling_min_freq" )
+            end
+            line = fp:read("*l")
+            if tonumber(line)/1E6 < min then
+                min = tonumber(line)/1E6
+            end
+            fp:close()
+        end
+    else
+        fp = io.open(sys_base_path .. "/" .. string.format("cpu%d",cpuid) .. "/cpufreq/scaling_min_freq")
+        if verbosity == 3 then
+            print_stdout("Reading "..sys_base_path .. "/" .. string.format("cpu%d",cpuid) .. "/cpufreq/scaling_min_freq" )
+        end
+        line = fp:read("*l")
+        if tonumber(line)/1E6 < min then
+            min = tonumber(line)/1E6
+        end
+        fp:close()
+    end
+    return min
+end
+
+function getCurrentMaxFreq(cpuid)
+    local max = 0
+    if cpuid == nil or cpuid < 0 then
+        for cpuid=0,topo["numHWThreads"]-1 do
+            fp = io.open(sys_base_path .. "/" .. string.format("cpu%d",cpuid) .. "/cpufreq/scaling_max_freq")
+            if verbosity == 3 then
+                print_stdout("Reading "..sys_base_path .. "/" .. string.format("cpu%d",cpuid) .. "/cpufreq/scaling_max_freq" )
+            end
+            line = fp:read("*l")
+            if tonumber(line)/1E6 > max then
+                max = tonumber(line)/1E6
+            end
+            fp:close()
+        end
+    else
+        fp = io.open(sys_base_path .. "/" .. string.format("cpu%d",cpuid) .. "/cpufreq/scaling_max_freq")
+        if verbosity == 3 then
+            print_stdout("Reading "..sys_base_path .. "/" .. string.format("cpu%d",cpuid) .. "/cpufreq/scaling_max_freq" )
+        end
+        line = fp:read("*l")
+        if tonumber(line)/1E6 > max then
+            max = tonumber(line)/1E6
+        end
+        fp:close()
+    end
+    return max
+end
+
+
 function getAvailFreq(cpuid)
-    if (cpuid == nil) or (cpuid < 1) then
+    if cpuid == nil then
         cpuid = 0
     end
-    local fp = io.open(sys_base_path .. "/" .. string.format("cpu%d",cpuid) .. "/cpufreq/scaling_available_frequencies")
-    if verbosity == 3 then
-        print("Reading "..sys_base_path .. "/" .. string.format("cpu%d",cpuid) .. "/cpufreq/scaling_available_frequencies" )
+    if cpuid < 0 then
+        cpuid = 0
     end
-    local line = fp:read("*l")
+    fp = io.open(sys_base_path .. "/" .. string.format("cpu%d",cpuid) .. "/cpufreq/scaling_available_frequencies")
+    if verbosity == 3 then
+        print_stdout("Reading "..sys_base_path .. "/" .. string.format("cpu%d",cpuid) .. "/cpufreq/scaling_available_frequencies" )
+    end
+    line = fp:read("*l")
     fp:close()
+    
     local tmp = likwid.stringsplit(line:gsub("^%s*(.-)%s*$", "%1"), " ", nil, " ")
     local avail = {}
-    local turbo = tostring(tonumber(tmp[1])/1E6)
+    local turbo = tonumber(tmp[1])/1E6
+    local j = 1
     for i=2,#tmp do
-        avail[i-1] = tostring(tonumber(tmp[i])/1E6)
-        if not avail[i-1]:match("%d.%d") then
-            avail[i-1] = avail[i-1] ..".0"
+        local freq = tonumber(tmp[i])/1E6
+        avail[j] = tostring(freq)
+        if not avail[j]:match("%d+.%d+") then
+            avail[j] = avail[j] ..".0"
         end
+        j = j + 1
     end
     if verbosity == 1 then
-        print(string.format("The system provides %d scaling frequencies, frequency %s is taken as turbo mode", #avail,turbo))
+        print_stdout(string.format("The system provides %d scaling frequencies, frequency %s is taken as turbo mode", #avail,turbo))
     end
-    return avail, turbo
+    return avail, tostring(turbo)
 end
 
 function getCurFreq()
@@ -87,7 +154,7 @@ function getCurFreq()
     for cpuid=0,topo["numHWThreads"]-1 do
         local fp = io.open(sys_base_path .. "/" .. string.format("cpu%d",cpuid) .. "/cpufreq/scaling_cur_freq")
         if verbosity == 3 then
-            print("Reading "..sys_base_path .. "/" .. string.format("cpu%d",cpuid) .. "/cpufreq/scaling_cur_freq" )
+            print_stdout("Reading "..sys_base_path .. "/" .. string.format("cpu%d",cpuid) .. "/cpufreq/scaling_cur_freq" )
         end
         local line = fp:read("*l")
         fp:close()
@@ -97,7 +164,7 @@ function getCurFreq()
         end
         local fp = io.open(sys_base_path .. "/" .. string.format("cpu%d",cpuid) .. "/cpufreq/scaling_governor")
         if verbosity == 3 then
-            print("Reading "..sys_base_path .. "/" .. string.format("cpu%d",cpuid) .. "/cpufreq/scaling_governor" )
+            print_stdout("Reading "..sys_base_path .. "/" .. string.format("cpu%d",cpuid) .. "/cpufreq/scaling_governor" )
         end
         local line = fp:read("*l")
         fp:close()
@@ -112,7 +179,7 @@ function getAvailGovs(cpuid)
     end
     local fp = io.open(sys_base_path .. "/" .. string.format("cpu%d",cpuid) .. "/cpufreq/scaling_available_governors")
     if verbosity == 3 then
-        print("Reading "..sys_base_path .. "/" .. string.format("cpu%d",cpuid) .. "/cpufreq/scaling_available_governors" )
+        print_stdout("Reading "..sys_base_path .. "/" .. string.format("cpu%d",cpuid) .. "/cpufreq/scaling_available_governors" )
     end
     local line = fp:read("*l")
     fp:close()
@@ -125,7 +192,7 @@ function getAvailGovs(cpuid)
     end
     table.insert(avail, "turbo")
     if verbosity == 1 then
-        print(string.format("The system provides %d scaling governors", #avail))
+        print_stdout(string.format("The system provides %d scaling governors", #avail))
     end
     return avail
 end
@@ -133,7 +200,7 @@ end
 local function testDriver()
     local fp = io.open(sys_base_path .. "/" .. string.format("cpu%d",0) .. "/cpufreq/scaling_driver")
     if verbosity == 3 then
-        print("Reading "..sys_base_path .. "/" .. string.format("cpu%d",0) .. "/cpufreq/scaling_driver" )
+        print_stdout("Reading "..sys_base_path .. "/" .. string.format("cpu%d",0) .. "/cpufreq/scaling_driver" )
     end
     local line = fp:read("*l")
     fp:close()
@@ -177,21 +244,24 @@ for opt,arg in likwid.getopt(arg, {"g:", "c:", "f:", "l", "p", "h", "v", "m", "h
     elseif (opt == "m") then
         printAvailGovs = true
     elseif opt == "?" then
-        print("Invalid commandline option -"..arg)
+        print_stderr("Invalid commandline option -"..arg)
+        os.exit(1)
+    elseif opt == "!" then
+        print_stderr("Option requires an argument")
         os.exit(1)
     end
 end
 if not testDriver() then
-    print("The system does not use the acpi-cpufreq driver, other drivers are not usable with likwid-setFrequencies.")
+    print_stderr("The system does not use the acpi-cpufreq driver, other drivers are not usable with likwid-setFrequencies.")
     os.exit(1)
 end
 
 topo = likwid.getCpuTopology()
 affinity = likwid.getAffinityInfo()
-if not domain then
+if not domain or domain == "N" then
     domain = "N:0-" .. tostring(topo["numHWThreads"]-1)
 end
-if domain:match("[SCM]%d+") then
+if domain:match("[SCM]%d") then
     for i, dom in pairs(affinity["domains"]) do
         if dom["tag"]:match(domain) then
             domain = domain..":0-"..tostring(dom["numberOfProcessors"]-1)
@@ -201,34 +271,43 @@ end
 cpulist = {}
 numthreads, cpulist = likwid.cpustr_to_cpulist(domain)
 if verbosity == 3 then
-    print(string.format("Given CPU expression expands to %d CPU cores:", numthreads))
+    print_stdout(string.format("Given CPU expression expands to %d CPU cores:", numthreads))
     local str = tostring(cpulist[1])
     for i=2, numthreads  do
         str = str .. "," .. tostring(cpulist[i])
     end
-    print(str)
+    print_stdout(str)
 end
 
 
 if printAvailGovs then
     local govs = getAvailGovs(nil)
-    print("Available governors:")
-    print(table.concat(govs, ", "))
+    print_stdout("Available governors:")
+    print_stdout(table.concat(govs, ", "))
 end
 
 if printAvailFreq then
+    print_stdout("Available frequencies:")
+    local out = {}
+    local i = 1;
     local freqs, turbo = getAvailFreq(nil)
-    print("Available frequencies:")
-    print(turbo .. ", " .. table.concat(freqs, ", "))
+    if turbo ~= "0" then
+        table.insert(out, turbo)
+    end
+    for i=1,#freqs do
+        table.insert(out, freqs[i])
+    end
+
+    print_stdout(table.concat(out, " "))
 end
 
 if printCurFreq then
-    print("Current frequencies:")
+    print_stdout("Current frequencies:")
     local freqs = {}
     local govs = {}
     freqs, govs = getCurFreq()
     for i=1,#cpulist do
-        print(string.format("CPU %d: governor %12s frequency %5s GHz",cpulist[i],govs[cpulist[i]], freqs[cpulist[i]]))
+        print_stdout(string.format("CPU %d: governor %12s frequency %5s GHz",cpulist[i],govs[cpulist[i]], freqs[cpulist[i]]))
     end
 end
 
@@ -236,33 +315,39 @@ if printAvailGovs or printAvailFreq or printCurFreq then
     os.exit(0)
 end
 
+if numthreads > 0 and not (frequency or governor) then
+    print_stderr("You need to set either a frequency or governor for the selected CPUs on commandline")
+    os.exit(1)
+end
+
 if frequency then
-    local freqs, turbo = getAvailFreq(nil)
-    local valid_freq = false
-    for k,v in pairs(freqs) do
-        if (frequency == v) then
-            valid_freq = true
-            break
-        end
-    end
-    if frequency == turbo then
-        valid_freq = true
-    end
-    if not valid_freq then
-        print(string.format("Frequency %s not available! Please select one of\n%s", frequency, table.concat(freqs, ", ")))
-        os.exit(1)
-    end
     for i=1,#cpulist do
+        local freqs, turbo = getAvailFreq(cpulist[i])
+        local valid_freq = false
+        for k,v in pairs(freqs) do
+            if (frequency == v) then
+                valid_freq = true
+                break
+            end
+        end
+        if frequency == turbo then
+            valid_freq = true
+        end
+        if not valid_freq then
+            print_stderr(string.format("Frequency %s not available for CPU %d! Please select one of\n%s", frequency, cpulist[i], table.concat(freqs, ", ")))
+            os.exit(1)
+        end
+    
         local cmd = set_command .. " " .. tostring(cpulist[i]) .. " " .. tostring(tonumber(frequency)*1E6)
         if governor then
             cmd = cmd .. " " .. governor
         end
         if verbosity == 3 then
-            print("Execute: ".. cmd)
+            print_stdout("Execute: ".. cmd)
         end
         local err = os.execute(cmd)
         if err == false or err == nil then
-            print("Failed to set frequency for CPU "..tostring(cpulist[i]))
+            print_stderr("Failed to set frequency for CPU "..tostring(cpulist[i]))
         end
     end
     if governor then
@@ -281,14 +366,14 @@ if governor then
             break
         end
     end
-    if (governor == "turbo") then
+    if governor == "turbo" and turbo ~= "0" then
         valid_gov = true
         for i=1,#cpulist do
             cur_freqs[cpulist[i]] = turbo
         end
     end
     if not valid_gov then
-        print(string.format("Governor %s not available! Please select one of\n%s", governor, table.concat(govs, ", ")))
+        print_stderr(string.format("Governor %s not available! Please select one of\n%s", governor, table.concat(govs, ", ")))
         os.exit(1)
     end
     for i=1,#cpulist do
@@ -300,11 +385,11 @@ if governor then
                 cmd = cmd .. tostring(tonumber(cur_freqs[cpulist[i]])*1E6) .. " " .. governor
             end
             if verbosity == 3 then
-                print("Execute: ".. cmd)
+                print_stdout("Execute: ".. cmd)
             end
             local err = os.execute(cmd)
             if err == false or err == nil then
-                print("Failed to set governor for CPU "..tostring(cpulist[i]))
+                print_stderr("Failed to set governor for CPU "..tostring(cpulist[i]))
             end
         end
     end
