@@ -5,13 +5,13 @@
  *
  *      Description:  Utility routines manipulating bit arrays.
  *
- *      Version:   4.0
- *      Released:  16.6.2015
+ *      Version:   4.1
+ *      Released:  13.6.2016
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -72,18 +72,18 @@ extractBitField(uint32_t inField, uint32_t width, uint32_t offset)
 uint32_t
 getBitFieldWidth(uint32_t number)
 {
-    uint32_t fieldWidth;
+    uint32_t fieldWidth=0;
 
     number--;
     if (number == 0)
     {
         return 0;
     }
-
+#ifdef __x86_64
     __asm__ volatile ( "bsr %%eax, %%ecx\n\t"
             : "=c" (fieldWidth)
             : "a"(number));
-
+#endif
 
     return fieldWidth+1;  /* bsr returns the position, we want the width */
 }
