@@ -6,13 +6,13 @@
  *      Description: Hashtable implementation based on SGLIB.
  *                   Used for Marker API result handling.
  *
- *      Version:   4.0
- *      Released:  16.6.2015
+ *      Version:   4.1
+ *      Released:  13.6.2016
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -161,6 +161,12 @@ void hashTable_finalize(int* numThreads, int* numRegions, LikwidResults** result
                 fprintf(stderr, "Failed to allocate %lu bytes for the count storage\n", numberOfThreads * sizeof(uint32_t));
                 break;
             }
+            (*results)[i].cpulist = (int*) malloc(numberOfThreads * sizeof(int));
+            if (!(*results)[i].count)
+            {
+                fprintf(stderr, "Failed to allocate %lu bytes for the cpulist storage\n", numberOfThreads * sizeof(int));
+                break;
+            }
             (*results)[i].counters = (double**) malloc(numberOfThreads * sizeof(double*));
             if (!(*results)[i].counters)
             {
@@ -172,6 +178,7 @@ void hashTable_finalize(int* numThreads, int* numRegions, LikwidResults** result
             {
                 (*results)[i].time[j] = 0.0;
                 (*results)[i].count[j] = 0;
+                (*results)[i].cpulist[j] = -1;
                 (*results)[i].counters[j] = (double*) malloc(NUM_PMC * sizeof(double));
                 if (!(*results)[i].counters)
                 {
@@ -224,17 +231,24 @@ void hashTable_finalize(int* numThreads, int* numRegions, LikwidResults** result
 
                 (*results)[*regionId].count[threadId] = threadResult->count;
                 (*results)[*regionId].time[threadId] = threadResult->time;
+                (*results)[*regionId].cpulist[threadId] = threadResult->cpuID;
 
                 for ( int j=0; j < NUM_PMC; j++ )
                 {
                     (*results)[*regionId].counters[threadId][j] = threadResult->PMcounters[j];
                 }
+                bdestroy(threadResult->label);
+                free(threadResult);
             }
 
             threadId++;
+            g_hash_table_destroy(resPtr->hashTable);
+            free(resPtr);
+            threadList[core] = NULL;
         }
     }
-
+    g_hash_table_destroy(regionLookup);
+    regionLookup = NULL;
     (*numThreads) = numberOfThreads;
     (*numRegions) = numberOfRegions;
 }
