@@ -5,13 +5,13 @@
  *
  *      Description:  Interface to hwloc for PCI device lookup
  *
- *      Version:   4.0
- *      Released:  16.6.2015
+ *      Version:   4.1
+ *      Released:  13.6.2016
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -35,7 +35,6 @@
 
 #include <hwloc.h>
 #include <types.h>
-#include <accessClient.h>
 #include <bstrlib.h>
 #include <affinity.h>
 #include <topology.h>
@@ -53,14 +52,14 @@ hwloc_pci_init(uint16_t testDevice, char** socket_bus, int* nrSockets)
 
     if (!hwloc_topology)
     {
-        hwloc_topology_init(&hwloc_topology);
-        hwloc_topology_set_flags(hwloc_topology, HWLOC_TOPOLOGY_FLAG_WHOLE_IO );
-        hwloc_topology_load(hwloc_topology);
+        likwid_hwloc_topology_init(&hwloc_topology);
+        likwid_hwloc_topology_set_flags(hwloc_topology, HWLOC_TOPOLOGY_FLAG_WHOLE_IO );
+        likwid_hwloc_topology_load(hwloc_topology);
     }
 
-    for(i = 0; i < hwloc_get_nbobjs_by_type(hwloc_topology, HWLOC_OBJ_PCI_DEVICE); i++)
+    for(i = 0; i < likwid_hwloc_get_nbobjs_by_type(hwloc_topology, HWLOC_OBJ_PCI_DEVICE); i++)
     {
-        obj = hwloc_get_obj_by_type(hwloc_topology, HWLOC_OBJ_PCI_DEVICE, i);
+        obj = likwid_hwloc_get_obj_by_type(hwloc_topology, HWLOC_OBJ_PCI_DEVICE, i);
         if (obj->attr->pcidev.vendor_id != testVendor)
         {
             continue;
