@@ -6,7 +6,7 @@
  *      Description:  Module implementing Intel RAPL interface
  *
  *      Version:   4.1
- *      Released:  19.5.2016
+ *      Released:  13.6.2016
  *
  *      Authors:  Jan Treibig (jt), jan.treibig@gmail.com,
  *                Thomas Roehl (tr), thomas.roehl@googlemail.com
@@ -80,6 +80,8 @@ power_init(int cpuId)
         case ATOM_SILVERMONT_Z1:
         case ATOM_SILVERMONT_Z2:
         case ATOM_SILVERMONT_F:
+        case ATOM_SILVERMONT_AIR:
+        case ATOM_SILVERMONT_GOLD:
         case BROADWELL:
         case BROADWELL_E:
         case BROADWELL_D:
