@@ -5,13 +5,13 @@
  *
  *      Description:  Module implementing Intel TM/TM2 interface
  *
- *      Version:   4.0
- *      Released:  16.6.2015
+ *      Version:   4.1
+ *      Released:  13.6.2016
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -49,6 +49,8 @@ ThermalInfo thermal_info;
 void thermal_init(int cpuId)
 {
     uint64_t flags=0ULL;
+    HPMinit();
+    HPMaddThread(cpuId);
 
     if ( cpuid_hasFeature(TM2) )
     {
