@@ -121,13 +121,11 @@ int main(int argn, char** argc)
     printf(HLINE);
 #pragma omp parallel
     {
-	LIKWID_MARKER_THREADINIT;
 #pragma omp master
 	{
 	    printf ("Number of Threads requested = %i\n",omp_get_num_threads());
 	}
-
-	printf ("Thread %d running on processor %d ....\n",omp_get_thread_num(),threadGetProcessorId());
+	printf ("Thread %d running on processor %d ....\n",omp_get_thread_num(),sched_getcpu());
     }
 #endif
 
@@ -145,9 +143,11 @@ int main(int argn, char** argc)
         for (int k=0; k<ITER; k++)
         {
             LIKWID_MARKER_START("copy");
+#pragma simd
 #pragma omp for
             for (int j=0; j<SIZE; j++)
             {
+            
                 c[j] = a[j];
             }
             LIKWID_MARKER_STOP("copy");
@@ -159,17 +159,21 @@ int main(int argn, char** argc)
     time_start(&timer);
 #pragma omp parallel
     {
+	LIKWID_MARKER_START("triad_total");
         for (int k=0; k<ITER; k++)
         {
 
             LIKWID_MARKER_START("triad");
+#pragma simd
 #pragma omp for
             for (int j=0; j<SIZE; j++)
             {
+
                 a[j] = b[j] +  c[j] * d[j];
             }
             LIKWID_MARKER_STOP("triad");
         }
+	LIKWID_MARKER_STOP("triad_total");
     }
     time_stop(&timer);
     triad_time = time_print(&timer)/(double)ITER;
@@ -186,6 +190,10 @@ int main(int argn, char** argc)
 
 
     LIKWID_MARKER_CLOSE;
+    free(a);
+    free(b);
+    free(c);
+    free(d);
     return 0;
 }
 
